#!/usr/bin/env python
# coding: utf-8
"""Simple wrapper for python multiprocessing."""

__url__ = "https://github.com/metaist/ezq"
__version__ = "1.0.0"
__pubdate__ = "unpublished"  # date/time in UTC
__author__ = "Metaist LLC"
__email__ = "metaist@metaist.com"
__copyright__ = "Copyright 2021 by Metaist LLC"
__license__ = "MIT"
